package cn.iocoder.yudao.module.bpm.controller.admin.oa;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.contract.BpmOAContractPageReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.contract.BpmOAContractRespVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.payment.BpmOAPaymentCreateReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.payment.BpmOAPaymentPageReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.payment.BpmOAPaymentRespVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.payment.FactoryDetailVO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.BpmOAPaymentDO;
import cn.iocoder.yudao.module.bpm.service.oa.BpmOAPaymentService;
import cn.iocoder.yudao.module.system.api.bank.BankApi;
import cn.iocoder.yudao.module.system.api.bank.dto.BankRespDTO;
import cn.iocoder.yudao.module.system.api.dept.DeptApi;
import cn.iocoder.yudao.module.system.api.dept.dto.DeptRespDTO;
import cn.iocoder.yudao.module.system.api.project.ProjectApi;
import cn.iocoder.yudao.module.system.api.project.dto.ProjectDTO;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.util.Strings;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.*;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.common.util.collection.CollectionUtils.convertList;
import static cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils.getLoginUserId;

/**
 * OA 付款申请 Controller
 *
 * @author 符溶馨

 */
@Tag(name = "管理后台 - OA 付款申请")
@RestController
@RequestMapping("/bpm/oa/payment")
@Validated
public class BpmOAPaymentController {

    @Resource
    private BpmOAPaymentService paymentService;

    @Resource
    private ProjectApi projectApi;

    @Resource
    private DeptApi deptApi;

    @Resource
    private BankApi bankApi;

    @PostMapping("/create")
    @Operation(summary = "创建付款申请")
    public CommonResult<Long> createPayment(@Valid @RequestBody BpmOAPaymentCreateReqVO createReqVO) {

        return success(paymentService.createPayment(getLoginUserId(), createReqVO));
    }

    @GetMapping("/get")
    @Operation(summary = "获得付款申请")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    public CommonResult<BpmOAPaymentRespVO> getPayment(@RequestParam("id") Long id) {

        BpmOAPaymentDO payment = paymentService.getPayment(id);
        BpmOAPaymentRespVO respVO = BeanUtils.toBean(payment, BpmOAPaymentRespVO.class);
        if (respVO != null) {

            // 获取部门信息Map
            List<Long> deptIds = new ArrayList<>();
            if (CollectionUtil.isNotEmpty(respVO.getFactoryDetail())) {
                deptIds.addAll(convertList(respVO.getFactoryDetail(), FactoryDetailVO::getDeptId));
            }else {
                deptIds.add(payment.getDeptId());
            }
            deptIds.add(payment.getPaymentCompany());
            Map<Long, DeptRespDTO> dtoMap = deptApi.getDeptMap(new HashSet<>(deptIds));

            // 设置部门名称
            respVO.setDeptName(dtoMap.get(payment.getDeptId()) != null ? dtoMap.get(payment.getDeptId()).getName() : null);
            // 设置付款公司名称
            respVO.setPaymentCompanyName(dtoMap.get(payment.getPaymentCompany()).getName());
            // 设置工厂名称
            respVO.getFactoryDetail().forEach(item -> {
                item.setFactoryName(dtoMap.get(item.getDeptId()).getName());
            });

            if (StringUtils.isNotEmpty(payment.getProjectNo())) {
                // 获取项目信息
                ProjectDTO project = projectApi.getProject(payment.getProjectNo()).getCheckedData();
                // 设置项目名称
                respVO.setProjectName(project.getName());
            }

            if (payment.getBankId() != null) {
                // 获取银行卡信息
                BankRespDTO bankResp = bankApi.getBank(payment.getBankId()).getCheckedData();
                // 设置银行卡信息
                respVO.setBankName(bankResp.getBankName());
                respVO.setBankNo(bankResp.getBankNo());
                respVO.setNickname(bankResp.getNickname());
            }
        }

        return success(respVO);
    }

    @GetMapping("/getByProcessInstanceId")
    @Operation(summary = "获得付款申请")
    @Parameter(name = "BpmOAPaymentRespVO", description = "流程实例编号", required = true, example = "1024")
    public CommonResult<BpmOAPaymentRespVO> getByProcessInstanceId(@RequestParam("processInstanceId") String processInstanceId) {

        BpmOAPaymentDO payment = paymentService.getByProcessInstanceId(processInstanceId);
        BpmOAPaymentRespVO respVO = BeanUtils.toBean(payment, BpmOAPaymentRespVO.class);
        if (respVO != null) {
            // 获取部门信息Map
            List<Long> deptIds = new ArrayList<>();
            if (CollectionUtil.isNotEmpty(respVO.getFactoryDetail())) {
                deptIds.addAll(convertList(respVO.getFactoryDetail(), FactoryDetailVO::getDeptId));
            }else {
                deptIds.add(payment.getDeptId());
            }
            deptIds.add(payment.getPaymentCompany());
            Map<Long, DeptRespDTO> dtoMap = deptApi.getDeptMap(new HashSet<>(deptIds));

            // 设置部门名称
            respVO.setDeptName(dtoMap.get(payment.getDeptId()) != null ? dtoMap.get(payment.getDeptId()).getName() : null);
            // 设置付款公司名称
            respVO.setPaymentCompanyName(dtoMap.get(payment.getPaymentCompany()).getName());
            // 设置工厂名称
            respVO.getFactoryDetail().forEach(item -> {
                item.setFactoryName(dtoMap.get(item.getDeptId()).getName());
            });

            if (payment.getProjectNo() != null) {
                // 获取项目信息
                ProjectDTO project = projectApi.getProject(payment.getProjectNo()).getCheckedData();
                // 设置项目名称
                respVO.setProjectName(project.getName());
            }

            if (payment.getBankId() != null) {
                // 获取银行卡信息
                BankRespDTO bankResp = bankApi.getBank(payment.getBankId()).getCheckedData();
                // 设置银行卡信息
                respVO.setBankName(bankResp.getBankName());
                respVO.setBankNo(bankResp.getBankNo());
                respVO.setNickname(bankResp.getNickname());
            }
        }

        return success(respVO);
    }

    @GetMapping("/get-list")
    @Operation(summary = "获得指定条件的付款申请")
    @Parameter(name = "type", description = "付款类型", required = true, example = "1")
    public CommonResult<List<BpmOAPaymentRespVO>> getPayment(@RequestParam("type") Integer type) {

        return success(paymentService.getPaymentList(type));
    }

    @GetMapping("/page")
    @Operation(summary = "获得 后补票的付款申请分页列表")
    public CommonResult<PageResult<BpmOAPaymentRespVO>> getPage(@Valid BpmOAPaymentPageReqVO pageReqVO) {

        return success(paymentService.getPaymentPage(pageReqVO));
    }
}