package cn.iocoder.yudao.module.bpm.service.oa;

import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.UploadUserFile;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.module.bpm.api.task.dto.BpmProcessInstanceCreateReqDTO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.payment.BpmOAPaymentCreateReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.payment.BpmOAPaymentPageReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.payment.BpmOAPaymentRespVO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.financialpayment.FinancialPaymentDO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.BpmOAPaymentDO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.BpmOAReimbursementItemDO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.task.BpmProcessInstanceExtDO;
import cn.iocoder.yudao.module.bpm.dal.mysql.oa.BpmOAPaymentMapper;
import cn.iocoder.yudao.module.bpm.enums.task.BpmProcessInstanceResultEnum;
import cn.iocoder.yudao.module.bpm.service.financialpayment.FinancialPaymentService;
import cn.iocoder.yudao.module.bpm.service.task.BpmHistoryProcessInstanceService;
import cn.iocoder.yudao.module.bpm.service.task.BpmProcessInstanceService;
import cn.iocoder.yudao.module.system.api.user.AdminUserApi;
import cn.iocoder.yudao.module.system.api.user.dto.AdminUserRespDTO;
import org.flowable.engine.runtime.ProcessInstance;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.bpm.enums.ErrorCodeConstants.OA_PAYMENT_NOT_EXISTS;

/**
 * OA 付款申请 Service 实现类
 *
 * @author 符溶馨
 */
@Service
@Validated
public class BpmOAPaymentServiceImpl extends BpmOABaseService implements BpmOAPaymentService{

    /**
     * OA 付款对应的流程定义 KEY
     */
    public static final String PROCESS_KEY = "oa_payment_2";

    @Resource
    private BpmOAPaymentMapper paymentMapper;

    @Resource
    private BpmProcessInstanceService processInstanceService;

    @Resource
    private BpmHistoryProcessInstanceService historyProcessInstanceService;

    @Resource
    private FinancialPaymentService financialPaymentService;

    @Resource
    private AdminUserApi userApi;

    @Override
    public Long createPayment(Long userId, BpmOAPaymentCreateReqVO createReqVO) {

        //插入OA 付款申请
        BpmOAPaymentDO paymentDO = BeanUtils.toBean(createReqVO, BpmOAPaymentDO.class).setUserId(userId)
                .setResult(BpmProcessInstanceResultEnum.PROCESS.getResult());
        paymentMapper.insert(paymentDO);

        // 发起 BPM 流程
        Map<String, Object> processInstanceVariables = new HashMap<>();
        String processInstanceId = processInstanceService.createProcessInstance(userId,
                new BpmProcessInstanceCreateReqDTO().setProcessDefinitionKey(PROCESS_KEY)
                        .setVariables(processInstanceVariables).setBusinessKey(String.valueOf(paymentDO.getId())));

        // 将工作流的编号，更新到 OA 付款申请单中
        paymentMapper.updateById(new BpmOAPaymentDO().setId(paymentDO.getId()).setProcessInstanceId(processInstanceId));

        // 判断是否为重新发起的流程
        if (createReqVO.getProcessInstanceId() != null && createReqVO.getResult() == 3) {

            historyProcessInstanceService.createHistoryProcessInstance(processInstanceId, createReqVO.getProcessInstanceId());
        }

        List<UploadUserFile> fileItems = createReqVO.getFileItems() ;
        //这里的逻辑，如果fileItems不为空，且有数据，那么说明是上传了附件的，则需要更工作流文件表对应的实例Id
        if (fileItems != null && !fileItems.isEmpty()) {
            uploadBpmFileProcessInstanceId(processInstanceId,fileItems) ;
        }
        return paymentDO.getId();
    }

    @Override
    public void updatePaymentResult(String processInstanceId, Long id, Integer result) {

        BpmOAPaymentDO paymentDO = validateLeaveExists(id);
        //审核通过 （最后节点）
        if (BpmProcessInstanceResultEnum.APPROVE.getResult().equals(result)) {

            ProcessInstance instance = processInstanceService.getProcessInstance(processInstanceId);

            if (instance.isEnded()) {

                BpmProcessInstanceExtDO processInstance = processInstanceService.getProcessInstanceDO(processInstanceId);
                // -- 插入到财务支付表中
                CommonResult<AdminUserRespDTO> user = userApi.getUser(paymentDO.getUserId());
                financialPaymentService.save(new FinancialPaymentDO()
                        .setUserId(paymentDO.getUserId())
                        .setDeptId(user.getData() == null ? null : user.getData().getDeptId())
                        .setProcessInstanceId(paymentDO.getProcessInstanceId())
                        .setReason(paymentDO.getReason())
                        .setObjectId(id)
                        .setType(4)
                        .setStatus(0)
                        .setAmountPayable(paymentDO.getAmount())
                        .setProcessInstanceName(processInstance.getName())
                        .setBeginTime(processInstance.getCreateTime())
                        .setEndTime(processInstance.getEndTime())
                );
            }
        }

        paymentMapper.updateById(new BpmOAPaymentDO().setId(id).setResult(result));
    }

    @Override
    public BpmOAPaymentDO getPayment(Long id) {

        return paymentMapper.selectById(id);
    }

    @Override
    public BpmOAPaymentDO getByProcessInstanceId(String processInstanceId) {

        return paymentMapper.selectOne(BpmOAPaymentDO::getProcessInstanceId, processInstanceId);
    }

    @Override
    public List<BpmOAPaymentRespVO> getPaymentList(Integer type) {

        return paymentMapper.selectPaymentList(type);
    }

    @Override
    public PageResult<BpmOAPaymentRespVO> getPaymentPage(BpmOAPaymentPageReqVO pageReqVO) {

        return null;
    }

    private BpmOAPaymentDO validateLeaveExists(Long id) {
        BpmOAPaymentDO paymentDO = paymentMapper.selectById(id);
        if (paymentDO == null) {
            throw exception(OA_PAYMENT_NOT_EXISTS);
        }
        return paymentDO;
    }
}
