package cn.iocoder.yudao.module.system.controller.admin.customersettlement;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import cn.iocoder.yudao.module.system.controller.admin.customersettlement.vo.CustomerSettlementPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.customersettlement.vo.CustomerSettlementRespVO;
import cn.iocoder.yudao.module.system.controller.admin.customersettlement.vo.CustomerSettlementSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.customersettlement.CustomerSettlementDO;
import cn.iocoder.yudao.module.system.service.customersettlement.CustomerSettlementService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.EXPORT;

@Tag(name = "管理后台 - 客户结算信息")
@RestController
@RequestMapping("/system/customer-settlement")
@Validated
public class CustomerSettlementController {

    @Resource
    private CustomerSettlementService customerSettlementService;

    @PostMapping("/create")
    @Operation(summary = "创建客户结算信息")
    @PreAuthorize("@ss.hasPermission('system:customer-settlement:create')")
    public CommonResult<Long> createCustomerSettlement(@Valid @RequestBody CustomerSettlementSaveReqVO createReqVO) {
        return success(customerSettlementService.createCustomerSettlement(createReqVO));
    }

    @PutMapping("/update")
    @Operation(summary = "更新客户结算信息")
    @PreAuthorize("@ss.hasPermission('system:customer-settlement:update')")
    public CommonResult<Boolean> updateCustomerSettlement(@Valid @RequestBody CustomerSettlementSaveReqVO updateReqVO) {
        customerSettlementService.updateCustomerSettlement(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除客户结算信息")
    @Parameter(name = "id", description = "编号", required = true)
    @PreAuthorize("@ss.hasPermission('system:customer-settlement:delete')")
    public CommonResult<Boolean> deleteCustomerSettlement(@RequestParam("id") Long id) {
        customerSettlementService.deleteCustomerSettlement(id);
        return success(true);
    }

    @GetMapping("/get")
    @Operation(summary = "获得客户结算信息")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    @PreAuthorize("@ss.hasPermission('system:customer-settlement:query')")
    public CommonResult<CustomerSettlementRespVO> getCustomerSettlement(@RequestParam("id") Long id) {
        CustomerSettlementDO customerSettlement = customerSettlementService.getCustomerSettlement(id);
        return success(BeanUtils.toBean(customerSettlement, CustomerSettlementRespVO.class));
    }

    @GetMapping("/page")
    @Operation(summary = "获得客户结算信息分页")
    @PreAuthorize("@ss.hasPermission('system:customer-settlement:query')")
    public CommonResult<PageResult<CustomerSettlementRespVO>> getCustomerSettlementPage(@Valid CustomerSettlementPageReqVO pageReqVO) {
        PageResult<CustomerSettlementDO> pageResult = customerSettlementService.getCustomerSettlementPage(pageReqVO);
        return success(BeanUtils.toBean(pageResult, CustomerSettlementRespVO.class));
    }

    @GetMapping("/export-excel")
    @Operation(summary = "导出客户结算信息 Excel")
    @PreAuthorize("@ss.hasPermission('system:customer-settlement:export')")
    @OperateLog(type = EXPORT)
    public void exportCustomerSettlementExcel(@Valid CustomerSettlementPageReqVO pageReqVO,
              HttpServletResponse response) throws IOException {
        pageReqVO.setPageSize(PageParam.PAGE_SIZE_NONE);
        List<CustomerSettlementDO> list = customerSettlementService.getCustomerSettlementPage(pageReqVO).getList();
        // 导出 Excel
        ExcelUtils.write(response, "客户结算信息.xls", "数据", CustomerSettlementRespVO.class,
                        BeanUtils.toBean(list, CustomerSettlementRespVO.class));
    }

}