package cn.iocoder.yudao.module.system.service.customersettlement;

import cn.iocoder.yudao.module.infra.api.file.FileApi;
import cn.iocoder.yudao.module.system.controller.admin.worklog.vo.upload.UploadUserFile;
import cn.iocoder.yudao.module.system.dal.dataobject.rental.RentalDepositRecordDO;
import org.apache.tomcat.util.digester.ObjectCreateRule;
import org.springframework.stereotype.Service;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

import cn.iocoder.yudao.module.system.controller.admin.customersettlement.vo.*;
import cn.iocoder.yudao.module.system.dal.dataobject.customersettlement.CustomerSettlementDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;

import cn.iocoder.yudao.module.system.dal.mysql.customersettlement.CustomerSettlementMapper;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.*;

/**
 * 客户结算信息 Service 实现类
 *
 * @author 陈宾顺
 */
@Service
@Validated
public class CustomerSettlementServiceImpl implements CustomerSettlementService {

    @Resource
    private CustomerSettlementMapper customerSettlementMapper;

    @Resource
    private FileApi fileApi;

    @Override
    public Long createCustomerSettlement(CustomerSettlementSaveReqVO createReqVO) {
        // 插入
        CustomerSettlementDO customerSettlement = BeanUtils.toBean(createReqVO, CustomerSettlementDO.class);
        customerSettlementMapper.insert(customerSettlement);

        // 更新交易凭证附件 业务编号
        UpdateBusinessFile(customerSettlement);
        // 返回
        return customerSettlement.getId();
    }

    private void UpdateBusinessFile(CustomerSettlementDO updateReqVO) {

        List<String> urls = updateReqVO.getUrl().stream().map(UploadUserFile::getUrl).collect(Collectors.toList());
        fileApi.updateBusinessFile(urls, updateReqVO.getId().toString());
    }

    @Override
    public void updateCustomerSettlement(CustomerSettlementSaveReqVO updateReqVO) {
        // 校验存在
        validateCustomerSettlementExists(updateReqVO.getId());
        // 更新
        CustomerSettlementDO updateObj = BeanUtils.toBean(updateReqVO, CustomerSettlementDO.class);
        customerSettlementMapper.updateById(updateObj);
    }

    @Override
    public void deleteCustomerSettlement(Long id) {
        // 校验存在
        validateCustomerSettlementExists(id);
        // 删除
        customerSettlementMapper.deleteById(id);
    }

    private void validateCustomerSettlementExists(Long id) {
        if (customerSettlementMapper.selectById(id) == null) {
            throw exception(CUSTOMER_SETTLEMENT_NOT_EXISTS);
        }
    }

    @Override
    public CustomerSettlementDO getCustomerSettlement(Long id) {
        return customerSettlementMapper.selectById(id);
    }

    @Override
    public PageResult<CustomerSettlementDO> getCustomerSettlementPage(CustomerSettlementPageReqVO pageReqVO) {
        return customerSettlementMapper.selectPage(pageReqVO);
    }

}