package cn.iocoder.yudao.module.system.service.customersettlement;

import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.mock.mockito.MockBean;

import javax.annotation.Resource;

import cn.iocoder.yudao.framework.test.core.ut.BaseDbUnitTest;

import cn.iocoder.yudao.module.system.controller.admin.customersettlement.vo.*;
import cn.iocoder.yudao.module.system.dal.dataobject.customersettlement.CustomerSettlementDO;
import cn.iocoder.yudao.module.system.dal.mysql.customersettlement.CustomerSettlementMapper;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

import javax.annotation.Resource;
import org.springframework.context.annotation.Import;
import java.util.*;
import java.time.LocalDateTime;

import static cn.hutool.core.util.RandomUtil.*;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.*;
import static cn.iocoder.yudao.framework.test.core.util.AssertUtils.*;
import static cn.iocoder.yudao.framework.test.core.util.RandomUtils.*;
import static cn.iocoder.yudao.framework.common.util.date.LocalDateTimeUtils.*;
import static cn.iocoder.yudao.framework.common.util.object.ObjectUtils.*;
import static cn.iocoder.yudao.framework.common.util.date.DateUtils.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

/**
 * {@link CustomerSettlementServiceImpl} 的单元测试类
 *
 * @author 陈宾顺
 */
@Import(CustomerSettlementServiceImpl.class)
public class CustomerSettlementServiceImplTest extends BaseDbUnitTest {

    @Resource
    private CustomerSettlementServiceImpl customerSettlementService;

    @Resource
    private CustomerSettlementMapper customerSettlementMapper;

    @Test
    public void testCreateCustomerSettlement_success() {
        // 准备参数
        CustomerSettlementSaveReqVO createReqVO = randomPojo(CustomerSettlementSaveReqVO.class).setId(null);

        // 调用
        Long customerSettlementId = customerSettlementService.createCustomerSettlement(createReqVO);
        // 断言
        assertNotNull(customerSettlementId);
        // 校验记录的属性是否正确
        CustomerSettlementDO customerSettlement = customerSettlementMapper.selectById(customerSettlementId);
        assertPojoEquals(createReqVO, customerSettlement, "id");
    }

    @Test
    public void testUpdateCustomerSettlement_success() {
        // mock 数据
        CustomerSettlementDO dbCustomerSettlement = randomPojo(CustomerSettlementDO.class);
        customerSettlementMapper.insert(dbCustomerSettlement);// @Sql: 先插入出一条存在的数据
        // 准备参数
        CustomerSettlementSaveReqVO updateReqVO = randomPojo(CustomerSettlementSaveReqVO.class, o -> {
            o.setId(dbCustomerSettlement.getId()); // 设置更新的 ID
        });

        // 调用
        customerSettlementService.updateCustomerSettlement(updateReqVO);
        // 校验是否更新正确
        CustomerSettlementDO customerSettlement = customerSettlementMapper.selectById(updateReqVO.getId()); // 获取最新的
        assertPojoEquals(updateReqVO, customerSettlement);
    }

    @Test
    public void testUpdateCustomerSettlement_notExists() {
        // 准备参数
        CustomerSettlementSaveReqVO updateReqVO = randomPojo(CustomerSettlementSaveReqVO.class);

        // 调用, 并断言异常
        assertServiceException(() -> customerSettlementService.updateCustomerSettlement(updateReqVO), CUSTOMER_SETTLEMENT_NOT_EXISTS);
    }

    @Test
    public void testDeleteCustomerSettlement_success() {
        // mock 数据
        CustomerSettlementDO dbCustomerSettlement = randomPojo(CustomerSettlementDO.class);
        customerSettlementMapper.insert(dbCustomerSettlement);// @Sql: 先插入出一条存在的数据
        // 准备参数
        Long id = dbCustomerSettlement.getId();

        // 调用
        customerSettlementService.deleteCustomerSettlement(id);
       // 校验数据不存在了
       assertNull(customerSettlementMapper.selectById(id));
    }

    @Test
    public void testDeleteCustomerSettlement_notExists() {
        // 准备参数
        Long id = randomLongId();

        // 调用, 并断言异常
        assertServiceException(() -> customerSettlementService.deleteCustomerSettlement(id), CUSTOMER_SETTLEMENT_NOT_EXISTS);
    }

    @Test
    @Disabled  // TODO 请修改 null 为需要的值，然后删除 @Disabled 注解
    public void testGetCustomerSettlementPage() {
       // mock 数据
       CustomerSettlementDO dbCustomerSettlement = randomPojo(CustomerSettlementDO.class, o -> { // 等会查询到
           o.setCustomerId(null);
           o.setBusinessType(null);
           o.setSettlementMonth(null);
           o.setPreSettlementAmount(null);
           o.setDeductionAmount(null);
           o.setSettlementAmount(null);
           o.setCompanyId(null);
           o.setUrl(null);
           o.setCreateTime(null);
       });
       customerSettlementMapper.insert(dbCustomerSettlement);
       // 测试 customerId 不匹配
       customerSettlementMapper.insert(cloneIgnoreId(dbCustomerSettlement, o -> o.setCustomerId(null)));
       // 测试 businessType 不匹配
       customerSettlementMapper.insert(cloneIgnoreId(dbCustomerSettlement, o -> o.setBusinessType(null)));
       // 测试 settlementMonth 不匹配
       customerSettlementMapper.insert(cloneIgnoreId(dbCustomerSettlement, o -> o.setSettlementMonth(null)));
       // 测试 preSettlementAmount 不匹配
       customerSettlementMapper.insert(cloneIgnoreId(dbCustomerSettlement, o -> o.setPreSettlementAmount(null)));
       // 测试 deductionAmount 不匹配
       customerSettlementMapper.insert(cloneIgnoreId(dbCustomerSettlement, o -> o.setDeductionAmount(null)));
       // 测试 settlementAmount 不匹配
       customerSettlementMapper.insert(cloneIgnoreId(dbCustomerSettlement, o -> o.setSettlementAmount(null)));
       // 测试 companyId 不匹配
       customerSettlementMapper.insert(cloneIgnoreId(dbCustomerSettlement, o -> o.setCompanyId(null)));
       // 测试 url 不匹配
       customerSettlementMapper.insert(cloneIgnoreId(dbCustomerSettlement, o -> o.setUrl(null)));
       // 测试 createTime 不匹配
       customerSettlementMapper.insert(cloneIgnoreId(dbCustomerSettlement, o -> o.setCreateTime(null)));
       // 准备参数
       CustomerSettlementPageReqVO reqVO = new CustomerSettlementPageReqVO();
       reqVO.setCustomerId(null);
       reqVO.setBusinessType(null);
       reqVO.setSettlementMonth(null);
       reqVO.setPreSettlementAmount(null);
       reqVO.setDeductionAmount(null);
       reqVO.setSettlementAmount(null);
       reqVO.setCompanyId(null);
       reqVO.setUrl(null);
       reqVO.setCreateTime(buildBetweenTime(2023, 2, 1, 2023, 2, 28));

       // 调用
       PageResult<CustomerSettlementDO> pageResult = customerSettlementService.getCustomerSettlementPage(reqVO);
       // 断言
       assertEquals(1, pageResult.getTotal());
       assertEquals(1, pageResult.getList().size());
       assertPojoEquals(dbCustomerSettlement, pageResult.getList().get(0));
    }

}